// https://www.youtube.com/watch?v=HwupNf9iCJk

use std::cell::{Cell, RefCell};

#[derive(Debug)]
// Could simplify this with Rc (see thread safe example)
struct Node<'a> {
    // Cell for copyable values
    val: Cell<i32>,
    // RefCell probably better as there is no copying but can panic
    str_val: RefCell<String>,
    adjacent: Vec<&'a Node<'a>>,
}

fn add_one(node: &Node) {
    let curr_val = node.val.get();
    node.val.set(curr_val + 1);
    for adj in node.adjacent.iter() {
        add_one(adj);
    }
}

fn add_urgency(node: &Node) {
    node.str_val.borrow_mut().push('!');
    for adj in node.adjacent.iter() {
        add_urgency(adj);
    }
}

fn main() {
    let a = Node {
        val: Cell::new(1),
        str_val: RefCell::new(String::from("aaa")),
        adjacent: vec![],
    };

    let b = Node {
        val: Cell::new(2),
        str_val: RefCell::new(String::from("bbb")),
        adjacent: vec![&a],
    };

    let c = Node {
        val: Cell::new(4),
        str_val: RefCell::new(String::from("ccc")),
        adjacent: vec![&a],
    };
    dbg!(&a);
    dbg!(&b);
    dbg!(&c);
    add_one(&b);
    dbg!(&b);
    add_urgency(&c);
    dbg!(&c);
}
